'''
peewee ORM Database Model definition 
Documentation: https://docs.peewee-orm.com

please look up model.dbml for Documentation
Online Viewer: https://dbdiagram.io 

'''


from peewee import *
from datetime import datetime

import json
from typing import TextIO
from pathlib import Path

db = DatabaseProxy()

# WIP: Add Switch Function
if True:
    database = SqliteDatabase(None, autoconnect=False)
else:
    database = PostgresqlDatabase(None, autoconnect=False)

db.initialize(database)

class BaseModel(Model):
    '''
    Base Model (needed for peewee) defines the Class Meta 
    and bounds Global db obj to every Table
    '''
    class Meta:
        database = db


class Town(BaseModel):
    '''
    Table for Storing Town Data 
    '''
    plz = IntegerField(primary_key=True)
    name = CharField()
    created_at = DateTimeField(default=datetime.now)


class Address(BaseModel):
    '''
    Table for Storing Address Data
    '''
    street = CharField()
    number = IntegerField()
    created_at = DateTimeField(default=datetime.now)


class Residence(BaseModel):
    '''
    Table for Storing a unique Postal Adress 
    by combining Towntable Data with Addresstable Data
    '''
    town_plz = ForeignKeyField(Town, backref='plz')
    address_id = ForeignKeyField(Address, backref='address')
    created_at = DateTimeField(default=datetime.now)


class Class(BaseModel):
    '''
    Baseline Order Base 
    
    Table for Storing a Class 
    '''
    name = CharField()
    created_at = DateTimeField(default=datetime.now)


class Group(BaseModel):
    '''
    Table for Storing a project Group 
    '''
    name = CharField()
    project = CharField()
    class_id = ForeignKeyField(Class, backref='class')
    created_at = DateTimeField(default=datetime.now)


class Student(BaseModel):
    '''
    Table for Storing a Student and linking him to appropiate Tables
    '''
    prename = CharField()
    surname = CharField()
    sex = CharField()
    class_id = ForeignKeyField(Class, backref='class')
    group_id = ForeignKeyField(Group, backref='group')
    grader = CharField()
    residence = ForeignKeyField(Residence, backref='residence', null=True)
    created_at = DateTimeField(default=datetime.now)


class Lecture(BaseModel):
    '''
    Table for defining a Lecture
    '''
    title = CharField()
    points = IntegerField()
    class_id = ForeignKeyField(Class, backref='class')
    created_at = DateTimeField(default=datetime.now)


class Submission(BaseModel):
    '''
    Table for defining a Submission from a Student for Lecture
    '''
    student_id = ForeignKeyField(Student, backref='student')
    lecture_id = ForeignKeyField(Lecture, backref='lecture')
    points = FloatField()
    created_at = DateTimeField(default=datetime.now)

